@@ -1,3 +1,13 @@
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2013-2015 Google, Inc.
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
+# Copyright (c) 2015 Mike Frysinger <vapier@gentoo.org>
+# Copyright (c) 2015 Mihai Balint <balint.mihai@gmail.com>
+# Copyright (c) 2015 Fabio Natali <me@fabionatali.com>
+# Copyright (c) 2015 Harut <yes@harutune.name>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -10,10 +20,10 @@ http://www.python.org/doc/essays/styleguide.html
 Some parts of the process_token method is based from The Tab Nanny std module.
 """
 
-from functools import reduce # pylint: disable=redefined-builtin
 import keyword
-import tokenize
 import sys
+import tokenize
+from functools import reduce # pylint: disable=redefined-builtin
 
 import six
 from six.moves import zip, map, filter # pylint: disable=redefined-builtin
@@ -337,7 +347,7 @@ class ContinuedLineState(object):
                 _Offsets(indentation + self._continuation_size, indentation),
                 _BeforeBlockOffsets(indentation + self._continuation_size,
                                     indentation + self._continuation_size * 2))
-        elif bracket == ':':
+        if bracket == ':':
             # If the dict key was on the same line as the open brace, the new
             # correct indent should be relative to the key instead of the
             # current indent level
@@ -352,13 +362,12 @@ class ContinuedLineState(object):
             # }
             # is handled by the special-casing for hanging continued string indents.
             return _ContinuedIndent(HANGING_DICT_VALUE, bracket, position, paren_align, next_align)
-        else:
-            return _ContinuedIndent(
-                HANGING,
-                bracket,
-                position,
-                _Offsets(indentation, indentation + self._continuation_size),
-                _Offsets(indentation + self._continuation_size))
+        return _ContinuedIndent(
+            HANGING,
+            bracket,
+            position,
+            _Offsets(indentation, indentation + self._continuation_size),
+            _Offsets(indentation + self._continuation_size))
 
     def _continuation_inside_bracket(self, bracket, pos):
         """Extracts indentation information for a continued indent."""
@@ -372,13 +381,12 @@ class ContinuedLineState(object):
                 pos,
                 _Offsets(token_start),
                 _BeforeBlockOffsets(next_token_start, next_token_start + self._continuation_size))
-        else:
-            return _ContinuedIndent(
-                CONTINUED,
-                bracket,
-                pos,
-                _Offsets(token_start),
-                _Offsets(next_token_start))
+        return _ContinuedIndent(
+            CONTINUED,
+            bracket,
+            pos,
+            _Offsets(token_start),
+            _Offsets(next_token_start))
 
     def pop_token(self):
         self._cont_stack.pop()
@@ -449,7 +457,7 @@ class FormatChecker(BaseTokenChecker):
                  'help': 'Maximum number of lines in a module'}
                ),
                ('indent-string',
-                {'default' : '    ', 'type' : "string", 'metavar' : '<string>',
+                {'default' : '    ', 'type' : "non_empty_string", 'metavar' : '<string>',
                  'help' : 'String used as indentation unit. This is usually '
                           '"    " (4 spaces) or "\\t" (1 tab).'}),
                ('indent-after-paren',
@@ -594,9 +602,32 @@ class FormatChecker(BaseTokenChecker):
 
         self._check_space(tokens, i, (policy_before, _IGNORE))
 
+    def _has_valid_type_annotation(self, tokens, i):
+        """Extended check of PEP-484 type hint presence"""
+        if not self._inside_brackets('('):
+            return False
+        bracket_level = 0
+        for token in tokens[i-1::-1]:
+            if token[1] == ':':
+                return True
+            if token[1] == '(':
+                return False
+            if token[1] == ']':
+                bracket_level += 1
+            elif token[1] == '[':
+                bracket_level -= 1
+            elif token[1] == ',':
+                if not bracket_level:
+                    return False
+            elif token[0] not in (tokenize.NAME, tokenize.STRING):
+                return False
+        return False
+
     def _check_equals_spacing(self, tokens, i):
         """Check the spacing of a single equals sign."""
-        if self._inside_brackets('(') or self._inside_brackets('lambda'):
+        if self._has_valid_type_annotation(tokens, i):
+            self._check_space(tokens, i, (_MUST, _MUST))
+        elif self._inside_brackets('(') or self._inside_brackets('lambda'):
             self._check_space(tokens, i, (_MUST_NOT, _MUST_NOT))
         else:
             self._check_space(tokens, i, (_MUST, _MUST))
@@ -638,23 +669,20 @@ class FormatChecker(BaseTokenChecker):
         def _policy_string(policy):
             if policy == _MUST:
                 return 'Exactly one', 'required'
-            else:
-                return 'No', 'allowed'
+            return 'No', 'allowed'
 
         def _name_construct(token):
             if token[1] == ',':
                 return 'comma'
-            elif token[1] == ':':
+            if token[1] == ':':
                 return ':'
-            elif token[1] in '()[]{}':
+            if token[1] in '()[]{}':
                 return 'bracket'
-            elif token[1] in ('<', '>', '<=', '>=', '!=', '=='):
+            if token[1] in ('<', '>', '<=', '>=', '!=', '=='):
                 return 'comparison'
-            else:
-                if self._inside_brackets('('):
-                    return 'keyword argument assignment'
-                else:
-                    return 'assignment'
+            if self._inside_brackets('('):
+                return 'keyword argument assignment'
+            return 'assignment'
 
         good_space = [True, True]
         token = tokens[i]
