@@ -1,20 +1,11 @@
-# Copyright (c) 2009-2010 Arista Networks, Inc. - James Lingard
-# Copyright (c) 2004-2013 LOGILAB S.A. (Paris, FRANCE).
-# Copyright 2012 Google Inc.
-#
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Copyright (c) 2009-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2013-2014 Google, Inc.
+# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Checker for string formatting operations.
 """
 
@@ -23,16 +14,14 @@ import tokenize
 import string
 import numbers
 
-import astroid
-from astroid import helpers
+import six
 
+import astroid
 from pylint.interfaces import ITokenChecker, IAstroidChecker, IRawChecker
 from pylint.checkers import BaseChecker, BaseTokenChecker
 from pylint.checkers import utils
 from pylint.checkers.utils import check_messages
 
-import six
-
 
 _PY3K = sys.version_info[:2] >= (3, 0)
 _PY27 = sys.version_info[:2] == (2, 7)
@@ -63,7 +52,7 @@ MSGS = {
     'W1301': ("Unused key %r in format string dictionary",
               "unused-format-string-key",
               "Used when a format string that uses named conversion specifiers \
-              is used with a dictionary that conWtains keys not required by the \
+              is used with a dictionary that contains keys not required by the \
               format string."),
     'E1304': ("Missing key %r in format string dictionary",
               "missing-format-string-key",
@@ -78,7 +67,10 @@ MSGS = {
               "too-few-format-args",
               "Used when a format string that uses unnamed conversion \
               specifiers is given too few arguments"),
-
+    'E1310': ("Suspicious argument in %s.%s call",
+              "bad-str-strip-call",
+              "The argument to a str.{l,r,}strip call contains a"
+              " duplicate character, "),
     'W1302': ("Invalid format string",
               "bad-format-string",
               "Used when a PEP 3101 format string is invalid.",
@@ -97,7 +89,7 @@ MSGS = {
     'W1305': ("Format string contains both automatic field numbering "
               "and manual field specification",
               "format-combined-specification",
-              "Usen when a PEP 3101 format string contains both automatic "
+              "Used when a PEP 3101 format string contains both automatic "
               "field numbering (e.g. '{}') and manual field "
               "specification (e.g. '{0}').",
               {'minversion': (2, 7)}),
@@ -120,7 +112,7 @@ OTHER_NODES = (astroid.Const, astroid.List, astroid.Repr,
                astroid.ListComp, astroid.SetComp, astroid.GeneratorExp)
 
 if _PY3K:
-    import _string
+    import _string # pylint: disable=wrong-import-position, wrong-import-order
 
     def split_format_field_names(format_string):
         return _string.formatter_field_name_split(format_string)
@@ -133,7 +125,10 @@ else:
                 yield is_attr, key
 
     def split_format_field_names(format_string):
-        keyname, fielditerator = format_string._formatter_field_name_split()
+        try:
+            keyname, fielditerator = format_string._formatter_field_name_split()
+        except ValueError:
+            raise utils.IncompleteFormatString
         # it will return longs, instead of ints, which will complicate
         # the output
         return keyname, _field_iterator_convertor(fielditerator)
@@ -193,7 +188,10 @@ def parse_format_method_string(format_string):
                 # to different output between 2 and 3
                 manual_pos_arg.add(str(keyname))
                 keyname = int(keyname)
-            keys.append((keyname, list(fielditerator)))
+            try:
+                keys.append((keyname, list(fielditerator)))
+            except ValueError:
+                raise utils.IncompleteFormatString()
         else:
             num_args += 1
     return keys, num_args, len(manual_pos_arg)
@@ -206,7 +204,7 @@ def get_args(callfunc):
     is the keyword arguments in a dict.
     """
     if callfunc.keywords:
-        named = {arg.arg: helpers.safe_infer(arg.value)
+        named = {arg.arg: utils.safe_infer(arg.value)
                  for arg in callfunc.keywords}
     else:
         named = {}
@@ -321,24 +319,14 @@ class StringFormatChecker(BaseChecker):
                     self.add_message('too-few-format-args', node=node)
 
 
-class StringMethodsChecker(BaseChecker):
-    __implements__ = (IAstroidChecker,)
-    name = 'string'
-    msgs = {
-        'E1310': ("Suspicious argument in %s.%s call",
-                  "bad-str-strip-call",
-                  "The argument to a str.{l,r,}strip call contains a"
-                  " duplicate character, "),
-        }
-
     @check_messages(*(MSGS.keys()))
     def visit_call(self, node):
-        func = helpers.safe_infer(node.func)
+        func = utils.safe_infer(node.func)
         if (isinstance(func, astroid.BoundMethod)
                 and isinstance(func.bound, astroid.Instance)
                 and func.bound.name in ('str', 'unicode', 'bytes')):
             if func.name in ('strip', 'lstrip', 'rstrip') and node.args:
-                arg = helpers.safe_infer(node.args[0])
+                arg = utils.safe_infer(node.args[0])
                 if not isinstance(arg, astroid.Const):
                     return
                 if len(arg.value) != len(set(arg.value)):
@@ -490,8 +478,8 @@ class StringMethodsChecker(BaseChecker):
                     warn_error = False
                     if hasattr(previous, 'getitem'):
                         try:
-                            previous = previous.getitem(specifier)
-                        except (IndexError, TypeError):
+                            previous = previous.getitem(astroid.Const(specifier))
+                        except (astroid.AstroidIndexError, astroid.AstroidTypeError):
                             warn_error = True
                         except astroid.InferenceError:
                             break
@@ -624,5 +612,4 @@ class StringConstantChecker(BaseTokenChecker):
 def register(linter):
     """required method to auto register this checker """
     linter.register_checker(StringFormatChecker(linter))
-    linter.register_checker(StringMethodsChecker(linter))
     linter.register_checker(StringConstantChecker(linter))
