@@ -1,15 +1,21 @@
-# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2011-2014 Google, Inc.
+# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
+# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """variables checkers for Python code
 """
+import copy
 import itertools
 import os
 import sys
 import re
-from copy import copy
 try:
     from functools import lru_cache
 except ImportError:
@@ -23,16 +29,12 @@ from astroid import modutils
 from pylint.interfaces import IAstroidChecker, INFERENCE, INFERENCE_FAILURE, HIGH
 from pylint.utils import get_global_option
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import (
-    PYMETHODS, is_ancestor_name, is_builtin,
-    is_defined_before, is_error, is_func_default, is_func_decorator,
-    assign_parent, check_messages, is_inside_except, clobber_in_except,
-    get_all_elements, has_known_bases, node_ignores_exception,
-    is_inside_abstract_class, is_comprehension, is_iterable,
-    safe_infer, is_from_fallback_block)
+from pylint.checkers import utils
 
 SPECIAL_OBJ = re.compile("^_{2}[a-z]+_{2}$")
 FUTURE = '__future__'
+# regexp for ignored argument name
+IGNORED_ARGUMENT_NAMES = re.compile('_.*')
 PY3K = sys.version_info >= (3, 0)
 
 
@@ -40,7 +42,7 @@ def _is_from_future_import(stmt, name):
     """Check if the name is a future import from another module."""
     try:
         module = stmt.do_import_module(stmt.modname)
-    except astroid.InferenceError:
+    except astroid.AstroidBuildingException:
         return
 
     for local_node in module.locals.get(name, []):
@@ -52,7 +54,9 @@ def _is_from_future_import(stmt, name):
 def in_for_else_branch(parent, stmt):
     """Returns True if stmt in inside the else branch for a parent For stmt."""
     return (isinstance(parent, astroid.For) and
-            any(else_stmt.parent_of(stmt) for else_stmt in parent.orelse))
+            any(else_stmt.parent_of(stmt) or else_stmt == stmt
+                for else_stmt in parent.orelse))
+
 
 @lru_cache(maxsize=1000)
 def overridden_method(klass, name):
@@ -212,6 +216,19 @@ def _import_name_is_global(stmt, global_names):
     return False
 
 
+def _flattened_scope_names(iterator):
+    values = (set(stmt.names) for stmt in iterator)
+    return set(itertools.chain.from_iterable(values))
+
+
+def _assigned_locally(name_node):
+    """
+    Checks if name_node has corresponding assign statement in same scope
+    """
+    assign_stmts = name_node.scope().nodes_of_class(astroid.AssignName)
+    return any(a.name == name_node.name for a in assign_stmts)
+
+
 MSGS = {
     'E0601': ('Using variable %r before assignment',
               'used-before-assignment',
@@ -242,7 +259,7 @@ MSGS = {
               'global-statement',
               'Used when you use the "global" statement to update a global \
               variable. Pylint just try to discourage this \
-              usage. That doesn\'t mean you can not use it !'),
+              usage. That doesn\'t mean you cannot use it !'),
     'W0604': ('Using the global statement at the module level', # W0103
               'global-at-module-level',
               'Used when you use the "global" statement at the module level \
@@ -316,18 +333,19 @@ class VariablesChecker(BaseChecker):
     priority = -1
     options = (("init-import",
                 {'default': 0, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'help' : 'Tells whether we should check for unused import in \
-__init__ files.'}),
+                 'help' : 'Tells whether we should check for unused import in '
+                          '__init__ files.'}),
                ("dummy-variables-rgx",
-                {'default': ('(_+[a-zA-Z0-9]*?$)|dummy'),
+                {'default': ('_+$|(_[a-zA-Z0-9_]*[a-zA-Z0-9]+?$)|dummy'),
                  'type' :'regexp', 'metavar' : '<regexp>',
-                 'help' : 'A regular expression matching the name of dummy \
-variables (i.e. expectedly not used).'}),
+                 'help' : 'A regular expression matching the name of dummy '
+                          'variables (i.e. expectedly not used).'}),
                ("additional-builtins",
                 {'default': (), 'type' : 'csv',
                  'metavar' : '<comma separated list>',
-                 'help' : 'List of additional names supposed to be defined in \
-builtins. Remember that you should avoid to define new builtins when possible.'
+                 'help' : 'List of additional names supposed to be defined in '
+                          'builtins. Remember that you should avoid to define new builtins '
+                          'when possible.'
                 }),
                ("callbacks",
                 {'default' : ('cb_', '_cb'), 'type' : 'csv',
@@ -342,11 +360,24 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                  'help': 'List of qualified module names which can have objects '
                          'that can redefine builtins.'}
                ),
+               ('ignored-argument-names',
+                {'default' : IGNORED_ARGUMENT_NAMES,
+                 'type' :'regexp', 'metavar' : '<regexp>',
+                 'help' : 'Argument names that match this expression will be '
+                          'ignored. Default to name with leading underscore'}
+               ),
+               ('allow-global-unused-variables',
+                {'default': True,
+                 'type': 'yn', 'metavar': '<y_or_n>',
+                 'help': 'Tells whether unused global variables should be treated as a violation.'}
+               ),
               )
+
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
         self._to_consume = None  # list of tuples: (to_consume:dict, consumed:dict, scope_type:str)
         self._checking_mod_attr = None
+        self._loop_variables = []
 
     # Relying on other checker's options, which might not have been initialized yet.
     @decorators.cachedproperty
@@ -357,21 +388,49 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def _ignored_modules(self):
         return get_global_option(self, 'ignored-modules', default=[])
 
+    @decorators.cachedproperty
+    def _allow_global_unused_variables(self):
+        return get_global_option(self, 'allow-global-unused-variables', default=True)
+
+    @utils.check_messages('redefined-outer-name')
+    def visit_for(self, node):
+        assigned_to = [var.name for var in node.target.nodes_of_class(astroid.AssignName)]
+
+        # Only check variables that are used
+        dummy_rgx = self.config.dummy_variables_rgx
+        assigned_to = [var for var in assigned_to if not dummy_rgx.match(var)]
+
+        for variable in assigned_to:
+            for outer_for, outer_variables in self._loop_variables:
+                if (variable in outer_variables
+                        and not in_for_else_branch(outer_for, node)):
+                    self.add_message(
+                        'redefined-outer-name',
+                        args=(variable, outer_for.fromlineno),
+                        node=node
+                    )
+                    break
+
+        self._loop_variables.append((node, assigned_to))
+
+    @utils.check_messages('redefined-outer-name')
+    def leave_for(self, _):
+        self._loop_variables.pop()
 
     def visit_module(self, node):
         """visit module : update consumption analysis variable
         checks globals doesn't overrides builtins
         """
-        self._to_consume = [(copy(node.locals), {}, 'module')]
+        self._to_consume = [(copy.copy(node.locals), {}, 'module')]
         for name, stmts in six.iteritems(node.locals):
-            if is_builtin(name) and not is_inside_except(stmts[0]):
+            if utils.is_builtin(name) and not utils.is_inside_except(stmts[0]):
                 if self._should_ignore_redefined_builtin(stmts[0]):
                     continue
                 self.add_message('redefined-builtin', args=name, node=stmts[0])
 
-    @check_messages('unused-import', 'unused-wildcard-import',
-                    'redefined-builtin', 'undefined-all-variable',
-                    'invalid-all-object')
+    @utils.check_messages('unused-import', 'unused-wildcard-import',
+                          'redefined-builtin', 'undefined-all-variable',
+                          'invalid-all-object', 'unused-variable')
     def leave_module(self, node):
         """leave module: check globals
         """
@@ -380,6 +439,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         # attempt to check for __all__ if defined
         if '__all__' in node.locals:
             self._check_all(node, not_consumed)
+
+        # check for unused globals
+        self._check_globals(not_consumed)
+
         # don't check unused imports in __init__ files
         if not self.config.init_import and node.package:
             return
@@ -434,6 +497,13 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                             # when the file will be checked
                             pass
 
+    def _check_globals(self, not_consumed):
+        if self._allow_global_unused_variables:
+            return
+        for name, nodes in six.iteritems(not_consumed):
+            for node in nodes:
+                self.add_message('unused-variable', args=(name,), node=node)
+
     def _check_imports(self, not_consumed):
         local_names = _fix_dot_imports(not_consumed)
         checked = set()
@@ -457,6 +527,8 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                         # Filter special objects (__doc__, __all__) etc.,
                         # because they can be imported for exporting.
                         continue
+                    if as_name == "_":
+                        continue
                     if as_name is None:
                         msg = "import %s" % imported_name
                     else:
@@ -490,7 +562,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_classdef(self, node):
         """visit class: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'class'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'class'))
 
     def leave_classdef(self, _):
         """leave class: update consumption analysis variable
@@ -501,7 +573,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_lambda(self, node):
         """visit lambda: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'lambda'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'lambda'))
 
     def leave_lambda(self, _):
         """leave lambda: update consumption analysis variable
@@ -512,7 +584,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_generatorexp(self, node):
         """visit genexpr: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'comprehension'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'comprehension'))
 
     def leave_generatorexp(self, _):
         """leave genexpr: update consumption analysis variable
@@ -523,7 +595,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_dictcomp(self, node):
         """visit dictcomp: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'comprehension'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'comprehension'))
 
     def leave_dictcomp(self, _):
         """leave dictcomp: update consumption analysis variable
@@ -534,7 +606,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_setcomp(self, node):
         """visit setcomp: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'comprehension'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'comprehension'))
 
     def leave_setcomp(self, _):
         """leave setcomp: update consumption analysis variable
@@ -545,13 +617,13 @@ builtins. Remember that you should avoid to define new builtins when possible.'
     def visit_functiondef(self, node):
         """visit function: update consumption analysis variable and check locals
         """
-        self._to_consume.append((copy(node.locals), {}, 'function'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'function'))
         if not (self.linter.is_message_enabled('redefined-outer-name') or
                 self.linter.is_message_enabled('redefined-builtin')):
             return
         globs = node.root().globals
         for name, stmt in node.items():
-            if is_inside_except(stmt):
+            if utils.is_inside_except(stmt):
                 continue
             if name in globs and not isinstance(stmt, astroid.Global):
                 definition = globs[name][0]
@@ -566,19 +638,22 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                     self.add_message('redefined-outer-name',
                                      args=(name, line), node=stmt)
 
-            elif is_builtin(name) and not self._should_ignore_redefined_builtin(stmt):
+            elif utils.is_builtin(name) and not self._should_ignore_redefined_builtin(stmt):
                 # do not print Redefining builtin for additional builtins
                 self.add_message('redefined-builtin', args=name, node=stmt)
 
-    @staticmethod
-    def _flattened_scope_names(iterator):
-        values = (set(stmt.names) for stmt in iterator)
-        return set(itertools.chain.from_iterable(values))
+    def _is_name_ignored(self, stmt, name):
+        authorized_rgx = self.config.dummy_variables_rgx
+        if (isinstance(stmt, astroid.AssignName)
+                and isinstance(stmt.parent, astroid.Arguments)):
+            regex = self.config.ignored_argument_names
+        else:
+            regex = authorized_rgx
+        return regex and regex.match(name)
 
     def _check_is_unused(self, name, node, stmt, global_names, nonlocal_names):
         # Ignore some special names specified by user configuration.
-        authorized_rgx = self.config.dummy_variables_rgx
-        if authorized_rgx.match(name):
+        if self._is_name_ignored(stmt, name):
             return
 
         # Ignore names imported by the global statement.
@@ -590,11 +665,14 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             if global_names and _import_name_is_global(stmt, global_names):
                 return
 
-        argnames = node.argnames()
+        argnames = list(itertools.chain(
+            node.argnames(),
+            [arg.name for arg in node.args.kwonlyargs]
+        ))
         is_method = node.is_method()
         klass = node.parent.frame()
         if is_method and isinstance(klass, astroid.ClassDef):
-            confidence = INFERENCE if has_known_bases(klass) else INFERENCE_FAILURE
+            confidence = INFERENCE if utils.has_known_bases(klass) else INFERENCE_FAILURE
         else:
             confidence = HIGH
 
@@ -608,12 +686,15 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 overridden = overridden_method(klass, node.name)
                 if overridden is not None and name in overridden.argnames():
                     return
-                if node.name in PYMETHODS and node.name not in ('__init__', '__new__'):
+                if node.name in utils.PYMETHODS and node.name not in ('__init__', '__new__'):
                     return
             # Don't check callback arguments
             if any(node.name.startswith(cb) or node.name.endswith(cb)
                    for cb in self.config.callbacks):
                 return
+            # Don't check arguments of singledispatch.register function.
+            if utils.is_registered_in_singledispatch_function(node):
+                return
             self.add_message('unused-argument', args=name, node=stmt,
                              confidence=confidence)
         else:
@@ -636,7 +717,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             return
 
         # Don't check arguments of function which are only raising an exception.
-        if is_error(node):
+        if utils.is_error(node):
             return
 
         # Don't check arguments of abstract methods or within an interface.
@@ -644,24 +725,24 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         if is_method and node.is_abstract():
             return
 
-        global_names = self._flattened_scope_names(node.nodes_of_class(astroid.Global))
-        nonlocal_names = self._flattened_scope_names(node.nodes_of_class(astroid.Nonlocal))
-
+        global_names = _flattened_scope_names(node.nodes_of_class(astroid.Global))
+        nonlocal_names = _flattened_scope_names(node.nodes_of_class(astroid.Nonlocal))
         for name, stmts in six.iteritems(not_consumed):
             self._check_is_unused(name, node, stmts[0], global_names, nonlocal_names)
 
     visit_asyncfunctiondef = visit_functiondef
     leave_asyncfunctiondef = leave_functiondef
 
-    @check_messages('global-variable-undefined', 'global-variable-not-assigned',
-                    'global-statement', 'global-at-module-level',
-                    'redefined-builtin')
+    @utils.check_messages('global-variable-undefined', 'global-variable-not-assigned',
+                          'global-statement', 'global-at-module-level',
+                          'redefined-builtin')
     def visit_global(self, node):
         """check names imported exists in the global scope"""
         frame = node.frame()
         if isinstance(frame, astroid.Module):
             self.add_message('global-at-module-level', node=node)
             return
+
         module = frame.root()
         default_message = True
         for name in node.names:
@@ -670,23 +751,16 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             except astroid.NotFoundError:
                 # unassigned global, skip
                 assign_nodes = []
-            for anode in assign_nodes:
-                if anode.parent is None:
-                    # node returned for builtin attribute such as __file__,
-                    # __doc__, etc...
-                    continue
-                if anode.frame() is frame:
-                    # same scope level assignment
-                    break
-            else:
-                if not _find_frame_imports(name, frame):
-                    self.add_message('global-variable-not-assigned',
-                                     args=name, node=node)
-                default_message = False
+
             if not assign_nodes:
+                self.add_message('global-variable-not-assigned',
+                                 args=name, node=node)
+                default_message = False
                 continue
+
             for anode in assign_nodes:
-                if anode.parent is None:
+                if (isinstance(anode, astroid.AssignName)
+                        and anode.name in module.special_attributes):
                     self.add_message('redefined-builtin', args=name, node=node)
                     break
                 if anode.frame() is module:
@@ -696,6 +770,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 # global undefined at the module scope
                 self.add_message('global-variable-undefined', args=name, node=node)
                 default_message = False
+
         if default_message:
             self.add_message('global-statement', node=node)
 
@@ -768,10 +843,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             return False
         return stmt.modname in self.config.redefining_builtins_modules
 
-    @check_messages('redefine-in-handler')
+    @utils.check_messages('redefine-in-handler')
     def visit_excepthandler(self, node):
-        for name in get_all_elements(node.name):
-            clobbering, args = clobber_in_except(name)
+        for name in utils.get_all_elements(node.name):
+            clobbering, args = utils.clobber_in_except(name)
             if clobbering:
                 self.add_message('redefine-in-handler', args=args, node=name)
 
@@ -826,7 +901,9 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         else:
             # we are in a local scope, check the name is not
             # defined in global or builtin scope
-            if defframe.root().lookup(name)[1]:
+            # skip this lookup if name is assigned later in function scope
+            forbid_lookup = isinstance(frame, astroid.FunctionDef) and _assigned_locally(node)
+            if not forbid_lookup and defframe.root().lookup(name)[1]:
                 maybee0601 = False
             else:
                 # check if we have a nonlocal
@@ -874,6 +951,15 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             maybee0601 = True
         else:
             maybee0601 = maybee0601 and stmt.fromlineno <= defstmt.fromlineno
+            if maybee0601 and stmt.fromlineno == defstmt.fromlineno:
+                if (isinstance(defframe, astroid.FunctionDef)
+                        and frame is defframe
+                        and defframe.parent_of(node)
+                        and stmt is not defstmt):
+                    # Single statement function, with the statement on the
+                    # same line as the function definition
+                    maybee0601 = False
+
         return maybee0601, annotation_return
 
     def _ignore_class_scope(self, node, name, frame):
@@ -903,7 +989,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                      in_annotation_or_default) and
                     name in frame_locals)
 
-    @check_messages(*(MSGS.keys()))
+    @utils.check_messages(*(MSGS.keys()))
     def visit_name(self, node):
         """check that a name is defined if the current scope and doesn't
         redefine a built-in
@@ -918,8 +1004,8 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         # if the name node is used as a function default argument's value or as
         # a decorator, then start from the parent frame of the function instead
         # of the function frame - and thus open an inner class scope
-        if (is_func_default(node) or is_func_decorator(node)
-                or is_ancestor_name(frame, node)):
+        if (utils.is_func_default(node) or utils.is_func_decorator(node)
+                or utils.is_ancestor_name(frame, node)):
             start_index = len(self._to_consume) - 2
         else:
             start_index = len(self._to_consume) - 1
@@ -941,7 +1027,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             # the name has already been consumed, only check it's not a loop
             # variable used outside the loop
             if name in consumed:
-                defnode = assign_parent(consumed[name][0])
+                defnode = utils.assign_parent(consumed[name][0])
                 self._check_late_binding_closure(node, defnode)
                 self._loopvar_name(node, name)
                 break
@@ -951,7 +1037,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             else:
                 continue
             # checks for use before assignment
-            defnode = assign_parent(to_consume[name][0])
+            defnode = utils.assign_parent(to_consume[name][0])
             if defnode is not None:
                 self._check_late_binding_closure(node, defnode)
                 defstmt = defnode.statement()
@@ -968,22 +1054,19 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                     base_scope_type, recursive_klass)
 
                 if (maybee0601
-                        and not is_defined_before(node)
-                        and not astroid.are_exclusive(stmt, defstmt, ('NameError',
-                                                                      'Exception',
-                                                                      'BaseException'))):
+                        and not utils.is_defined_before(node)
+                        and not astroid.are_exclusive(stmt, defstmt, ('NameError',))):
 
                     # Used and defined in the same place, e.g `x += 1` and `del x`
                     defined_by_stmt = (
                         defstmt is stmt
                         and isinstance(node, (astroid.DelName, astroid.AssignName))
                     )
-
                     if (recursive_klass
                             or defined_by_stmt
                             or annotation_return
                             or isinstance(defstmt, astroid.Delete)):
-                        if not node_ignores_exception(node, NameError):
+                        if not utils.node_ignores_exception(node, NameError):
                             self.add_message('undefined-variable', args=name,
                                              node=node)
                     elif base_scope_type != 'lambda':
@@ -1018,15 +1101,15 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         else:
             # we have not found the name, if it isn't a builtin, that's an
             # undefined name !
-            if not (name in astroid.Module.scope_attrs or is_builtin(name)
+            if not (name in astroid.Module.scope_attrs or utils.is_builtin(name)
                     or name in self.config.additional_builtins):
-                if not node_ignores_exception(node, NameError):
+                if not utils.node_ignores_exception(node, NameError):
                     self.add_message('undefined-variable', args=name, node=node)
 
-    @check_messages('no-name-in-module')
+    @utils.check_messages('no-name-in-module')
     def visit_import(self, node):
         """check modules attribute accesses"""
-        if not self._analyse_fallback_blocks and is_from_fallback_block(node):
+        if not self._analyse_fallback_blocks and utils.is_from_fallback_block(node):
             # No need to verify this, since ImportError is already
             # handled by the client code.
             return
@@ -1039,10 +1122,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 continue
             self._check_module_attrs(node, module, parts[1:])
 
-    @check_messages('no-name-in-module')
+    @utils.check_messages('no-name-in-module')
     def visit_importfrom(self, node):
         """check modules attribute accesses"""
-        if not self._analyse_fallback_blocks and is_from_fallback_block(node):
+        if not self._analyse_fallback_blocks and utils.is_from_fallback_block(node):
             # No need to verify this, since ImportError is already
             # handled by the client code.
             return
@@ -1050,7 +1133,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         name_parts = node.modname.split('.')
         try:
             module = node.do_import_module(name_parts[0])
-        except Exception:
+        except astroid.AstroidBuildingException:
             return
         module = self._check_module_attrs(node, module, name_parts[1:])
         if not module:
@@ -1060,7 +1143,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 continue
             self._check_module_attrs(node, module, name.split('.'))
 
-    @check_messages('unbalanced-tuple-unpacking', 'unpacking-non-sequence')
+    @utils.check_messages('unbalanced-tuple-unpacking', 'unpacking-non-sequence')
     def visit_assign(self, node):
         """Check unbalanced tuple unpacking for assignments
         and unpacking non-sequences.
@@ -1070,7 +1153,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
 
         targets = node.targets[0].itered()
         try:
-            infered = safe_infer(node.value)
+            infered = utils.safe_infer(node.value)
             if infered is not None:
                 self._check_unpacking(infered, node, targets)
         except astroid.InferenceError:
@@ -1080,9 +1163,9 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         """ Check for unbalanced tuple unpacking
         and unpacking non sequences.
         """
-        if is_inside_abstract_class(node):
+        if utils.is_inside_abstract_class(node):
             return
-        if is_comprehension(node):
+        if utils.is_comprehension(node):
             return
         if infered is astroid.YES:
             return
@@ -1105,7 +1188,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                                        len(values)))
         # attempt to check unpacking may be possible (ie RHS is iterable)
         else:
-            if not is_iterable(infered):
+            if not utils.is_iterable(infered):
                 self.add_message('unpacking-non-sequence', node=node,
                                  args=(_get_unpacking_extra_info(node, infered),))
 
@@ -1136,7 +1219,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         if module_names:
             # FIXME: other message if name is not the latest part of
             # module_names ?
-            modname = module and module.name or '__dict__'
+            modname = module.name if module else '__dict__'
             self.add_message('no-name-in-module', node=node,
                              args=('.'.join(module_names), modname))
             return None
@@ -1152,7 +1235,7 @@ class VariablesChecker3k(VariablesChecker):
     def visit_listcomp(self, node):
         """visit dictcomp: update consumption analysis variable
         """
-        self._to_consume.append((copy(node.locals), {}, 'comprehension'))
+        self._to_consume.append((copy.copy(node.locals), {}, 'comprehension'))
 
     def leave_listcomp(self, _):
         """leave dictcomp: update consumption analysis variable
@@ -1160,56 +1243,68 @@ class VariablesChecker3k(VariablesChecker):
         # do not check for not used locals here
         self._to_consume.pop()
 
+    def leave_functiondef(self, node):
+        self._check_metaclasses(node)
+        super(VariablesChecker3k, self).leave_functiondef(node)
+
     def leave_module(self, node):
-        """ Update consumption analysis variable
-        for metaclasses.
-        """
-        module_locals = self._to_consume[0][0]
-        module_imports = self._to_consume[0][1]
-        consumed = {}
+        self._check_metaclasses(node)
+        super(VariablesChecker3k, self).leave_module(node)
 
-        for klass in node.nodes_of_class(astroid.ClassDef):
-            found = metaclass = name = None
-            if not klass._metaclass:
-                # Skip if this class doesn't use
-                # explictly a metaclass, but inherits it from ancestors
-                continue
+    def _check_metaclasses(self, node):
+        """ Update consumption analysis for metaclasses. """
+        consumed = []  # [(scope_locals, consumed_key)]
 
-            metaclass = klass.metaclass()
+        for child_node in node.get_children():
+            if isinstance(child_node, astroid.ClassDef):
+                consumed.extend(self._check_classdef_metaclasses(child_node, node))
 
-            # Look the name in the already found locals.
-            # If it's not found there, look in the module locals
-            # and in the imported modules.
+        # Pop the consumed items, in order to avoid having
+        # unused-import and unused-variable false positives
+        for scope_locals, name in consumed:
+            scope_locals.pop(name, None)
+
+    def _check_classdef_metaclasses(self, klass, parent_node):
+        if not klass._metaclass:
+            # Skip if this class doesn't use explictly a metaclass, but inherits it from ancestors
+            return []
+
+        consumed = []  # [(scope_locals, consumed_key)]
+        metaclass = klass.metaclass()
+
+        name = None
+        if isinstance(klass._metaclass, astroid.Name):
+            name = klass._metaclass.name
+        elif metaclass:
+            name = metaclass.root().name
+
+        found = None
+        if name:
+            # check enclosing scopes starting from most local
+            for scope_locals, _, _ in self._to_consume[::-1]:
+                found = scope_locals.get(name)
+                if found:
+                    consumed.append((scope_locals, name))
+                    break
+
+        if found is None and not metaclass:
+            name = None
             if isinstance(klass._metaclass, astroid.Name):
                 name = klass._metaclass.name
-            elif metaclass:
-                # if it uses a `metaclass=module.Class`
-                name = metaclass.root().name
+            elif isinstance(klass._metaclass, astroid.Attribute):
+                name = klass._metaclass.as_string()
 
-            if name:
-                found = consumed.setdefault(
-                    name, module_locals.get(name, module_imports.get(name)))
+            if name is not None:
+                if not (name in astroid.Module.scope_attrs or
+                        utils.is_builtin(name) or
+                        name in self.config.additional_builtins or
+                        name in parent_node.locals):
+                    self.add_message('undefined-variable',
+                                     node=klass,
+                                     args=(name,))
 
-            if found is None and not metaclass:
-                name = None
-                if isinstance(klass._metaclass, astroid.Name):
-                    name = klass._metaclass.name
-                elif isinstance(klass._metaclass, astroid.Attribute):
-                    name = klass._metaclass.as_string()
+        return consumed
 
-                if name is not None:
-                    if not (name in astroid.Module.scope_attrs or
-                            is_builtin(name) or
-                            name in self.config.additional_builtins or
-                            name in node.locals):
-                        self.add_message('undefined-variable',
-                                         node=klass,
-                                         args=(name, ))
-        # Pop the consumed items, in order to
-        # avoid having unused-import false positives
-        for name in consumed:
-            module_locals.pop(name, None)
-        super(VariablesChecker3k, self).leave_module(node)
 
 if sys.version_info >= (3, 0):
     VariablesChecker = VariablesChecker3k
