@@ -1,5 +1,8 @@
-# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2014-2015 Bruno Daniel <bruno.daniel@blue-yonder.com>
+# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -31,7 +34,7 @@ class DocstringParameterChecker(BaseChecker):
 
     Activate this checker by adding the line::
 
-        load-plugins=pylint.extensions.check_docs
+        load-plugins=pylint.extensions.docparams
 
     to the ``MASTER`` section of your ``.pylintrc``.
 
@@ -40,7 +43,7 @@ class DocstringParameterChecker(BaseChecker):
     """
     __implements__ = IAstroidChecker
 
-    name = 'docstring_params'
+    name = 'parameter_documentation'
     msgs = {
         'W9003': ('"%s" missing or differing in parameter documentation',
                   'missing-param-doc',
@@ -54,32 +57,57 @@ class DocstringParameterChecker(BaseChecker):
         'W9006': ('"%s" not documented as being raised',
                   'missing-raises-doc',
                   'Please document exceptions for all raised exception types.'),
-        'W9007': ('Missing return type documentation',
-                  'missing-returns-doc',
-                  'Please add documentation about what this method returns.'),
         'W9008': ('Redundant returns documentation',
                   'redundant-returns-doc',
-                  'Please remove the return documentation from this method.'),
+                  'Please remove the return/rtype documentation from this method.'),
+        'W9010': ('Redundant yields documentation',
+                  'redundant-yields-doc',
+                  'Please remove the yields documentation from this method.'),
+        'W9011': ('Missing return documentation',
+                  'missing-return-doc',
+                  'Please add documentation about what this method returns.',
+                  {'old_names': [('W9007', 'missing-returns-doc')]}),
+        'W9012': ('Missing return type documentation',
+                  'missing-return-type-doc',
+                  'Please document the type returned by this method.',
+                  # we can't use the same old_name for two different warnings
+                  # {'old_names': [('W9007', 'missing-returns-doc')]},
+                 ),
+        'W9013': ('Missing yield documentation',
+                  'missing-yield-doc',
+                  'Please add documentation about what this generator yields.',
+                  {'old_names': [('W9009', 'missing-yields-doc')]}),
+        'W9014': ('Missing yield type documentation',
+                  'missing-yield-type-doc',
+                  'Please document the type yielded by this method.',
+                  # we can't use the same old_name for two different warnings
+                  # {'old_names': [('W9009', 'missing-yields-doc')]},
+                 ),
     }
 
     options = (('accept-no-param-doc',
                 {'default': True, 'type' : 'yn', 'metavar' : '<y or n>',
                  'help': 'Whether to accept totally missing parameter '
-                         'documentation in a docstring of a function that has '
+                         'documentation in the docstring of a function that has '
                          'parameters.'
                 }),
                ('accept-no-raise-doc',
                 {'default': True, 'type' : 'yn', 'metavar' : '<y or n>',
-                 'help': 'Whether to accept totally missing raises'
-                         'documentation in a docstring of a function that'
+                 'help': 'Whether to accept totally missing raises '
+                         'documentation in the docstring of a function that '
                          'raises an exception.'
                 }),
                ('accept-no-return-doc',
                 {'default': True, 'type' : 'yn', 'metavar' : '<y or n>',
-                 'help': 'Whether to accept totally missing return'
-                         'documentation in a docstring of a function that'
+                 'help': 'Whether to accept totally missing return '
+                         'documentation in the docstring of a function that '
                          'returns a statement.'
                 }),
+               ('accept-no-yields-doc',
+                {'default': True, 'type' : 'yn', 'metavar': '<y or n>',
+                 'help': 'Whether to accept totally missing yields '
+                         'documentation in the docstring of a generator.'
+                }),
               )
 
     priority = -2
@@ -96,6 +124,7 @@ class DocstringParameterChecker(BaseChecker):
         node_doc = utils.docstringify(node.doc)
         self.check_functiondef_params(node, node_doc)
         self.check_functiondef_returns(node, node_doc)
+        self.check_functiondef_yields(node, node_doc)
 
     def check_functiondef_params(self, node, node_doc):
         node_allow_no_param = None
@@ -107,8 +136,16 @@ class DocstringParameterChecker(BaseChecker):
 
                 # __init__ or class docstrings can have no parameters documented
                 # as long as the other documents them.
-                node_allow_no_param = class_doc.has_params() or None
-                class_allow_no_param = node_doc.has_params() or None
+                node_allow_no_param = (
+                    class_doc.has_params() or
+                    class_doc.params_documented_elsewhere() or
+                    None
+                )
+                class_allow_no_param = (
+                    node_doc.has_params() or
+                    node_doc.params_documented_elsewhere() or
+                    None
+                )
 
                 self.check_arguments_in_docstring(
                     class_doc, node.args, class_node, class_allow_no_param)
@@ -117,13 +154,26 @@ class DocstringParameterChecker(BaseChecker):
             node_doc, node.args, node, node_allow_no_param)
 
     def check_functiondef_returns(self, node, node_doc):
+        if not node_doc.supports_yields and node.is_generator():
+            return
+
         return_nodes = node.nodes_of_class(astroid.Return)
-        if (node_doc.has_returns() and
+        if ((node_doc.has_returns() or node_doc.has_rtype()) and
                 not any(utils.returns_something(ret_node) for ret_node in return_nodes)):
             self.add_message(
                 'redundant-returns-doc',
                 node=node)
 
+    def check_functiondef_yields(self, node, node_doc):
+        if not node_doc.supports_yields:
+            return
+
+        if ((node_doc.has_yields() or node_doc.has_yields_type()) and
+                not node.is_generator()):
+            self.add_message(
+                'redundant-yields-doc',
+                node=node)
+
     def visit_raise(self, node):
         func_node = node.frame()
         if not isinstance(func_node, astroid.FunctionDef):
@@ -157,10 +207,47 @@ class DocstringParameterChecker(BaseChecker):
 
         if not doc.has_returns():
             self.add_message(
-                'missing-returns-doc',
+                'missing-return-doc',
                 node=func_node
             )
 
+        if not doc.has_rtype():
+            self.add_message(
+                'missing-return-type-doc',
+                node=func_node
+            )
+
+    def visit_yield(self, node):
+        func_node = node.frame()
+        if not isinstance(func_node, astroid.FunctionDef):
+            return
+
+        doc = utils.docstringify(func_node.doc)
+        if not doc.is_valid() and self.config.accept_no_yields_doc:
+            return
+
+        if doc.supports_yields:
+            doc_has_yields = doc.has_yields()
+            doc_has_yields_type = doc.has_yields_type()
+        else:
+            doc_has_yields = doc.has_returns()
+            doc_has_yields_type = doc.has_rtype()
+
+        if not doc_has_yields:
+            self.add_message(
+                'missing-yield-doc',
+                node=func_node
+            )
+
+        if not doc_has_yields_type:
+            self.add_message(
+                'missing-yield-type-doc',
+                node=func_node
+            )
+
+    def visit_yieldfrom(self, node):
+        self.visit_yield(node)
+
     def check_arguments_in_docstring(self, doc, arguments_node, warning_node,
                                      accept_no_param_doc=None):
         """Check that all parameters in a function, method or class constructor
